/*
 * audio_decoder_mp3.h: The mp3 decoder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id: audio_decoder_mp3.h 256 2009-03-31 22:59:57Z hitman_47 $
 */

#ifndef __AUDIO_DECODER_MP3_H
#define __AUDIO_DECODER_MP3_H

#include <string.h>
#include <stdint.h>
#include <mad.h>
#include "config.h"
#include "audio_buffer.h"
#include "audio_madresampler.h"

/**
 *
 * cMP3Decoder opens an iStream (filestream or netstream) and
 * puts the data pieces on the mad_stream_decoder.
 * The mad_decoder will fill the output with PCM data.
 * PCM data must be resampled.
 * That PCM data will be placed on the audio_buffer (SimpleRingBuffer),
 * which can be read by the audio_player.
 *
 * ISTREAM ---> MAD_DECODER ---> MAD_RESAMPLER ---> PCMBuffer
 */
class cMp3Decoder
{
public:
	cMp3Decoder(void);
	virtual ~cMp3Decoder();
	unsigned long GetData(uint8_t * data, const unsigned long count);
	/**
	 * GetIndex
	 * @param: reference to the current position of the current track
	 * @return: errorcode
	 */
	int GetIndex(int & Current, int & Total, bool SnapToIFrame = false);
private:
	void Resample(mad_pcm & pcm);
	signed int ScaleTo16Bit(mad_fixed_t sample);
	int FillReadBuffer();
	/**
	 * DecodeNextChunk loads data from mp3-source and
	 * decodes that data.
	 * After that, it calls MadResample and in the latter function,
	 * the pointers according to the decoded data will be set.
	 */
	void DecodeNextChunk();
	virtual unsigned long GetDataFromSource(uint8_t * data, const unsigned long count) = 0;
private:
	static const int readBufferSize = READBUFFSIZE;
	MadResampler resampler;
	unsigned int sampleRate;
	unsigned char readBuffer[readBufferSize];
	struct mad_stream madStream;
	struct mad_frame madFrame;
	struct mad_synth madSynth;
	mad_fixed_t * leftBuffer;
	mad_fixed_t * rightBuffer;
	mad_fixed_t * leftBufferReadPtr;
	mad_fixed_t * rightBufferReadPtr;
	int samplesInBuffer;
	mad_timer_t timeIndex;
	cSimpleRingBuffer * mo_PCMBuffer;
	int m_sourcedataavailable;
};

#endif // __AUDIO_DECODER_H
