/*
 * audio_player.c: The audio player and control interface
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id: audio_player.c 256 2009-03-31 22:59:57Z hitman_47 $
 */

#include <sstream> // cWebServiceConnectorScrobbler::Submission
#include <limits.h>
#include <time.h>
#include <vdr/status.h>
#include "audio_player.h"
#include "audio_encap.h"
#include "config.h"
#include "audio_control.h"
#include <vdr/tools.h>
#include "logdefs.h"

cAudioPlayer::cAudioPlayer(void)
	: cPlayer(pmAudioOnly)
{
	SetDescription(PLUGIN_NAME "-AudioPlayer");
	mpo_ringBufferFrame = new cRingBufferFrame(PLAYER_BUFSIZE);
	m_playMode = pmNone;
	mpo_playFrame = 0;
	mpo_readFrame = 0;
	httpmp3decoder = 0;
	int ret = 0;

	switch(ret = Initialize()) {
		case 1:
			Skins.Message(mtError, tr("Handshake unsuccessful!; no such user -> means username or password_md5 are wrong"), 3);
			Quit();
			break;
		case 2:
			Skins.Message(mtError, tr("Handshake unsuccessful! Network failure"), 3);
			Quit();
			break;
		case 3:
			Skins.Message(mtError, tr("Received no data"), 3);
			Quit();
			break;
		case 4:
			Skins.Message(mtError, tr("No webservices available; cannot love, ban, etc."), 3);
			// Quit(); do not quit, not critical
			break;
	}

	Start();
}


cAudioPlayer::~cAudioPlayer(void)
{
	Cancel(MAXIMUM_THREADKILLTIME);
	Stop();
	Detach();

	if (httpmp3decoder)
		DELETENULL(httpmp3decoder);

	if (mpo_ringBufferFrame)
		DELETENULL(mpo_ringBufferFrame);
}


void cAudioPlayer::Action(void)
{
	uint8_t * b = NULL, * data = new uint8_t[MAX_FRAME_PAYLOAD];
	int bc = 0;
	cEncapsulator oEncapsulator;
	cPoller oPoller;

	mpo_readFrame = 0;
	oEncapsulator.SetPesParameters(0xBD, 0x8700);


	cDevice::PrimaryDevice()->SetCurrentAudioTrack(ttAudio);

/////	while (Running() && (m_playMode != pmStopped || !DeviceFlush(100))) {
	while (Running()) {
		h.Select(0, SOCKETS_SELECT_US); // pump network, will load data from net and buffer to NET_BUFFER
		LOCK_THREAD;
		if (!httpmp3decoder) {
			Next();
/////		} else if (DevicePoll(oPoller, POLLTIMEOUT) ) { // Put data on Dvbdevice:
} else if (DevicePoll(oPoller) ) { // Put data on Dvbdevice:

			// Frame to be read next time from source will be requested:
			if (!mpo_readFrame) {
				// Get DATA from source:
				int len = httpmp3decoder->GetData(data, MAX_FRAME_PAYLOAD);
				if (!len) {
#ifdef LASTFM_DEBUG_AUDIO
					LOGDBG("cAudioPlayer::Action: if (!len)\r\n");
#endif
					if (!httpmp3decoder->IsConnected()) {
#ifdef LASTFM_DEBUG_AUDIO
						LOGDBG("cAudioPlayer::Action: if (!httpmp3decoder->IsConnected())\r\n");
#endif
						Next();
					} else {
						static int cnt_bufferunderruns = 0;

						if (cnt_bufferunderruns > 2) {
							cnt_bufferunderruns = 0;
							Next();
						}
						cnt_bufferunderruns++;

						time_t time_lastconnloss = time(0);
						while (time(0) - time_lastconnloss < PLAYER_FAILURETIMEOUTINTERVAL) {
#ifdef LASTFM_DEBUG_AUDIO
							LOGDBG("cAudioPlayer::Action: buffering\r\n");
#endif
							h.Select(PLAYER_FAILURETIMEOUTINTERVAL, 0);
							cCondWait::SleepMs(3);
						}
						Clear();
					}
				} else if (len) {
					oEncapsulator.SetPcmParameters(2, 44100);
					bc = oEncapsulator.Data((uint8_t *)data, len);
					bc = oEncapsulator.ToPcm((uint8_t * *)&b);
					bc = oEncapsulator.ToPes((uint8_t * *)&b);

					mpo_readFrame = new cFrame(b, bc, ftUnknown);
					b = NULL;
					bc = 0;
				}
			}

			// If a new source frame has been found, it will be moved to the RingBuffer:
			if (mpo_readFrame) {
				if (mpo_ringBufferFrame->Put(mpo_readFrame))
					mpo_readFrame = NULL;
			}
			// playFrame is the Frame to be played with DVBoutput:
			if (!mpo_playFrame) {
				mpo_playFrame = mpo_ringBufferFrame->Get();
				b = NULL;
				bc = 0;
			}

			// when we have got a new playFrame, we will move it to the PESPLAYER (DVBouptut):
			if (mpo_playFrame) {
				if (!b) {
					b = mpo_playFrame->Data();
					bc = mpo_playFrame->Count();
				}
				if (b) {
					int i = 0;

					i = PlayPes(b, bc, false);

					if (0 < i) {
						b += i;
						bc -= i;
					}
				}
				if (!bc) {
					mpo_ringBufferFrame->Drop(mpo_playFrame);
					mpo_playFrame = NULL;
					b = NULL;
				}
			}


		} // if (DevicePoll ...
		cCondWait::SleepMs(3); // avoid high cpuload

	}                       // while (Running())
}


void cAudioPlayer::WaitPlayMode(ePlayMode test, bool negate, ePlayMode wait)
{
	mo_playerMutex.Lock();
	// negate == false: test == m_playMode
	// negate == true : test != m_playMode

	if ( (test == m_playMode) ^ negate) {
		while (m_playMode != wait && m_playMode != pmStopped) {
			mo_playerCond.Wait(mo_playerMutex);
		}
	}

	mo_playerMutex.Unlock();
}


void cAudioPlayer::SetPlayMode(ePlayMode mode)
{
	mo_playerMutex.Lock();
	m_playMode = mode;
	mo_playerCond.Broadcast();
	mo_playerMutex.Unlock();
}


void cAudioPlayer::Clear(void)
{
	LOCK_THREAD;
	SetPlayMode(pmNone);
	mpo_ringBufferFrame->Clear();
	mpo_readFrame = NULL;
	mpo_playFrame = NULL;
	DeviceClear();
}


void cAudioPlayer::Reset(void)
{
	LOCK_THREAD;
	Clear();
}


void cAudioPlayer::Play(void)
{
	SetPlayMode(pmPlay);
	DevicePlay();
}


void cAudioPlayer::Pause(void)
{
	if (pmPaused == m_playMode) {
		Play();
	} else {
		SetPlayMode(pmPaused);
		DeviceFreeze();
	}
}


void cAudioPlayer::Stop(void)
{
	// SetPlayMode() must be called after Reset()
	Reset();
	SetPlayMode(pmStopped);
}


void cAudioPlayer::Next(void)
{
	LOCK_THREAD;
Stop();
	std::string host("");
	port_t port = 0;

	// Get new songurl:
	std::string url(GetNextsongurl());

	if (httpmp3decoder)
		DELETENULL(httpmp3decoder);

	if (url.compare("") == 0)
		return;

	httpmp3decoder = new cHttpMp3Decoder(h);

	// we cAudioPlayer handles construction and destruction of httpmp3decoder
	httpmp3decoder->SetDeleteByHandler(false);
	httpmp3decoder->SetUrl(url);
	httpmp3decoder->Url(url, host, port);
	httpmp3decoder->Open(host, port);

	h.Add(httpmp3decoder);
	h.Select();         // wait until first event happened
}


bool cAudioPlayer::GetIndex(int & Current, int & Total, bool SnapToIFrame)
{
	if (httpmp3decoder)
		return httpmp3decoder->GetIndex(Current, Total);

	return false;
}

void cAudioPlayer::Quit(void) {
		Cancel(MAXIMUM_THREADKILLTIME);
		Stop();
		Detach();
		cControl::Shutdown();
}

