/**
 * setup_parameters.c: Setup submenu for changing user parameters
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id: setup_parameters.c 256 2009-03-31 22:59:57Z hitman_47 $
 */

#include "setup_parameters.h"
#include "utils.h"
#include "settings_xml.h"
#include "setup_favourites.h"

extern std::map<std::string, std::string> ParameterMap;
extern std::list<std::string> FavouriteList;

cSetupParameters::cSetupParameters(const std::string profilename)
	: m_profilename(profilename)
{
	// Load certain profile:
	std::map<std::string, std::string> ParameterMap;

	cXmlconfig::Load(profilename, ParameterMap, m_FavouriteList);

	if (!ParameterMap[ "Username" ].compare("") )   // no username defined
		ParameterMap[ "Username" ] = profilename;

	strncpy(newUsername, ParameterMap[ "Username" ].c_str(), sizeof(newUsername) - 1);
	newReceivermode = atoi(ParameterMap[ "Receivermode" ].c_str() );
	newAlbumcovermode = atoi(ParameterMap[ "Albumcovermode" ].c_str() );
	newOsd_Mode = atoi(ParameterMap[ "Osd_Mode" ].c_str() );
	newOsd_Autostartmode = atoi(ParameterMap[ "Osd_Autostartmode" ].c_str() );
	newOsd_Refreshinterval = atoi(ParameterMap[ "Osd_Refreshinterval" ].c_str() );
	newScrobblemode = atoi(ParameterMap[ "Scrobblemode" ].c_str() );

	// StraItem-Arrays:
	m_array_Receivermode[0] = (char *)trVDR("Button$Off");
	m_array_Receivermode[1] = (char *)tr("SoftdeviceReceiver");
	m_array_Osd_Mode[0] = (char *)tr("graphical");
	m_array_Osd_Mode[1] = (char *)tr("textual");

	SetHelp(0, 0, 0, 0);
	BuildOsdItems();
}


void cSetupParameters::BuildOsdItems(void)
{
	Clear();
	Add(new cOsdItem(hk(tr("Favourites...") ) ) );

	///newpassword leeren, damit es nicht im Menue als Klartext angezeigt wird.
	memset(&newPassphrase, 0, sizeof(newPassphrase) );

	Add(new cMenuEditStrItem(tr("Username"),
				 newUsername, sizeof( newUsername ), tr(cUtils::GetFileNameChars_lastfm()) ) );
	Add(new cMenuEditStrItem(tr("Passphrase"),
				 newPassphrase, sizeof( newPassphrase ), tr(cUtils::GetFileNameChars_lastfm()) ) );
#ifdef LASTFM_HAVE_MAGICK
	Add(new cMenuEditBoolItem(tr("Show albumcover?"), &newAlbumcovermode) );
#endif  //LASTFM_HAVE_MAGICK
	Add(new cMenuEditStraItem(tr("Osdmode"), &newOsd_Mode, 2, m_array_Osd_Mode) );
	Add(new cMenuEditIntItem(tr("Osd refresh interval"), &newOsd_Refreshinterval, 0, 60) );
	Add(new cMenuEditBoolItem(tr("Scrobblemode"), &newScrobblemode) );
	Add(new cMenuEditBoolItem(tr("Show Osd at startup?"), &newOsd_Autostartmode) );
}


void cSetupParameters::Store(void)
{
	// check, whether we edit an active user using this plugin, and fill the
	// parameters according to his profile:
	if (ParameterMap["CurrentProfileName"].compare(m_profilename) == 0) {
		std::map<std::string, std::string> ParameterMap;

		ParameterMap[ "Username" ] = newUsername;

		// Passwort nicht mitspeichern, wenn es leer gelassen wurde.
		if (strcmp(newPassphrase, "") != 0) {
			ParameterMap[ "Passphrase_md5" ] = cUtils::MD5(newPassphrase);
		}

		// reset password
		strcpy(newPassphrase, "");
#ifdef LASTFM_HAVE_MAGICK
		ParameterMap[ "Albumcovermode" ] = itoa(newAlbumcovermode);
#endif          //LASTFM_HAVE_MAGICK
		ParameterMap[ "Osd_Mode" ] = itoa(newOsd_Mode);
		ParameterMap[ "Osd_Refreshinterval" ] = itoa(newOsd_Refreshinterval);
		ParameterMap[ "Osd_Autostartmode" ] = itoa(newOsd_Autostartmode);
		ParameterMap[ "Scrobblemode" ] = itoa(newScrobblemode);

		// Store the data to the XML configfile:
		cXmlconfig::Save(m_profilename, ParameterMap, m_FavouriteList);
	} else {
/////		if (ParameterMap["Username"].compare(newUsername))
			ParameterMap[ "Username" ] = newUsername;


		// Passwort nicht mitspeichern, wenn es leer gelassen wurde.
		if (strcmp(newPassphrase, "") != 0) {
			ParameterMap[ "Passphrase_md5" ] = cUtils::MD5(newPassphrase);
		}

		// reset password
		strcpy(newPassphrase, "");

/**
 * Set parameters live into ParameterMap, used by current profile:
 */
#ifdef LASTFM_HAVE_MAGICK
		if (ParameterMap[ "Albumcovermode" ].compare(itoa(newAlbumcovermode) ) )
			ParameterMap[ "Albumcovermode" ] = itoa(newAlbumcovermode);
#endif          //LASTFM_HAVE_MAGICK
		if (ParameterMap[ "Osd_Mode" ].compare(itoa(newOsd_Mode) ) )
			ParameterMap[ "Osd_Mode" ] = itoa(newOsd_Mode);

		if (ParameterMap[ "Osd_Refreshinterval" ].compare(itoa(newOsd_Refreshinterval) ) )
			ParameterMap[ "Osd_Refreshinterval" ]
				= itoa(newOsd_Refreshinterval);

		if (ParameterMap[ "Osd_Autostartmode" ].compare(itoa(newOsd_Autostartmode) ) )
			ParameterMap[ "Osd_Autostartmode" ] = itoa(newOsd_Autostartmode);

		if (ParameterMap[ "Scrobblemode" ].compare(itoa(newScrobblemode) ) )
			ParameterMap[ "Scrobblemode" ] = itoa(newScrobblemode);

		FavouriteList = m_FavouriteList;

		// Store the data to the XML configfile:
		cXmlconfig::Save(m_profilename, ParameterMap, m_FavouriteList);
	}
}


eOSState cSetupParameters::ProcessKey(eKeys Key)
{
	eOSState state = cOsdMenu::ProcessKey(Key);

	switch (state) {
	/** when user presses kBack, setup shall store even then,
	 * because of the favourites */
	case osBack:
		Store();
		break;

	case osUnknown:
		if (Key == kOk) {
			if (!strcmp(Get(Current())->Text(), tr("Favourites...") ) ) {
				return AddSubMenu(new cSetupFavourites(m_FavouriteList) );
			} else {
				Store();
				state = osBack;
			}
		}
		break;

	default:
		break;
	}

	return state;
}


